/*********************************************************************/
/*                                                                   */
/*  This Program Written by Paul Edwards.                            */
/*  Released to the Public Domain                                    */
/*                                                                   */
/*********************************************************************/
/*********************************************************************/
/*                                                                   */
/*  freqcrc - compute the CRC required to perform a Yoohoo request   */
/*  for the details given in the hello structure below.              */
/*  The user will be prompted to enter a node, name and system name. */
/*                                                                   */
/*  Note that the BBS name and sysop name are blank-padded, whilst   */
/*  the password is NUL-padded.  Blanks are easier for script        */
/*  writing, whilst NULs are better for appearing on the other       */
/*  sysop's log (ie that way it doesn't come up and say "unexpected  */
/*  password '        ').                                            */
/*                                                                   */
/*********************************************************************/

#include <stdio.h>
#include <string.h>

#include "crcxm.h"

static char *strinl(char *work);

typedef struct 
{
  char fixed1[2];
  char fixed2[2];
  char prodcode[2];
  char major[2];
  char minor[2];
  char sysname[60];
  char sysop[20];
  char zone[2];
  char net[2];
  char node[2];
  char point[2];
  char password[8];
  char reserved1[8];
  char capability[2];
  char reserved2[12];
} hello;

int main(void)
{
  hello h;
  unsigned char *buffer;
  char inbuf[200];
  crcxm crc;
  int x;
  int zone, net, node, point;
  int len;

  printf("\n");
  printf("Devil Dialer CRC calculator\n");
  memset(&h,0x00,sizeof h);
  h.fixed1[0] = 0x6f;
  h.fixed1[1] = 0x00;
  h.fixed2[0] = 0x01;
  h.fixed2[1] = 0x00;
  h.prodcode[0] = 0x00;
  h.prodcode[1] = 0x00;
  h.major[0] = 0x00;
  h.major[1] = 0x00;
  h.minor[0] = 0x00;
  h.minor[1] = 0x00;
  strcpy(h.sysname,
         "Devil Dialer                                               ");
  strcpy(h.sysop, "The Devil          ");
  h.zone[0] = 0x9a;
  h.zone[1] = 0x02;
  h.net[0] = 0x9a;
  h.net[1] = 0x02;
  h.node[0] = 0x9a;
  h.node[1] = 0x02;
  h.point[0] = 0x00;
  h.point[1] = 0x00;
  memset(h.password,0x00,sizeof h.password);
  memset(h.reserved1,0x00,sizeof h.reserved1);
  h.capability[0] = 0x04;
  h.capability[1] = 0x00;
  memset(h.reserved2,0x00,sizeof h.reserved2);
  printf("\n");
  printf("what is your node number, e.g. 666:666/666?\n");
  fgets(inbuf, sizeof inbuf, stdin);
  strinl(inbuf);
  if (strlen(inbuf) != 0)
  {
    if (strchr(inbuf, '.') == NULL)
    {
      sscanf(inbuf, "%d:%d/%d", &zone, &net, &node);
      point = 0;
    }
    else
    {
      sscanf(inbuf, "%d:%d/%d.%d", &zone, &net, &node, &point);
    }
    h.zone[0] = zone & 0xff;
    h.zone[1] = (zone >> 8) & 0xff;
    h.net[0] = net & 0xff;
    h.net[1] = (net >> 8) & 0xff;
    h.node[0] = node & 0xff;
    h.node[1] = (node >> 8) & 0xff;
    h.point[0] = point & 0xff;
    h.point[1] = (point >> 8) & 0xff;
    printf("Your address is %d:%d/%d.%d\n", zone, net, node, point);
  }
  else 
  {
    printf("Your address is 666:666/666.0\n");
  }	

  printf("\n");
  printf("what is your name, e.g. John Smith?\n");
  fgets(inbuf, sizeof inbuf, stdin);
  strinl(inbuf);
  len = strlen(inbuf);
  if (len != 0)
  {
    if (len >= sizeof h.sysop)
    {
      len = sizeof h.sysop - 1;
    }
    memcpy(h.sysop, inbuf, len);
    memset(h.sysop+len, ' ', sizeof h.sysop - len - 1);
  }
  printf("your name is %s\n", h.sysop);

  printf("\n");
  printf("what is your system's name, e.g. The Biggest BBS Ever?\n");
  fgets(inbuf, sizeof inbuf, stdin);
  strinl(inbuf);
  len = strlen(inbuf);
  if (len != 0)
  {
    if (len >= sizeof h.sysname)
    {
      len = sizeof h.sysname - 1;
    }
    memcpy(h.sysname, inbuf, len);
    memset(h.sysname+len, ' ', sizeof h.sysname - len - 1);
  }
  printf("your system is called %s\n", h.sysname);
  
  printf("\n");
  printf("what password does your boss use (this is only of interest\n");
  printf("to points, normally just press enter to leave blank\n");
  fgets(inbuf, sizeof inbuf, stdin);
  strinl(inbuf);
  len = strlen(inbuf);
  if (len != 0)
  {
    if (len >= sizeof h.password)
    {
      len = sizeof h.password;
    }
    memcpy(h.password, inbuf, len);
    memset(h.password+len, '\0', sizeof h.password - len);
  }
  printf("your password is \"%.8s\"\n", h.password);
  buffer = (unsigned char *)&h;  
  crcxm_init(crc);
  for (x=0;x<128;x++)
      crcxm_update(crc, buffer[x]);
  printf("\n");
  printf("The CRC you need is %0.4x\n",crc);
  
  return (0);
}

static char *strinl(char *work)
{
  int i;
  
  i = strlen(work);
  if (i > 0)
  {
    if (*(work+i-1) == '\n') *(work+i-1) = '\0';
  }
  return (work);
}
